#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Test the events policy."""
from os.path import join
from json import loads
from cherrypy.test import helper
from common_test import CommonCPSetup


class TestEventsPolicy(helper.CPWebCase, CommonCPSetup):
    """Test the Events policy service."""

    PORT = 8181
    HOST = '127.0.0.1'
    headers = [('Content-Type', 'application/json')]

    def test_events_query(self):
        """Test posting the queries."""
        valid_query = loads(
            open(join('test_files', 'events_query.json')).read())
        ret_data = self.get_json_page('/events/dmlb2001', valid_query)
        self.assertFalse(ret_data is None)
        self.assertTrue('status' in ret_data)
        self.assertEqual(ret_data['status'], 'success')



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Data release policy for command line tools."""
from __future__ import print_function
from datetime import datetime
from json import dumps
from six import text_type
import requests
from dateutil import parser
from .config import get_config
from .admin import AdminPolicy
from .search.base import SearchBase

VALID_KEYWORDS = [
    'projects.actual_end_date',
    'projects.actual_start_date',
    'projects.submitted_date',
    'projects.accepted_date',
    'projects.closed_date',
    'transactions.created',
    'transactions.updated'
]


def collate_objs_from_key(resp, objs, date_key):
    """Deduplicate objs and make sure they have dates."""
    for chk_obj in resp.json():
        if chk_obj['_id'] not in objs.keys() and chk_obj.get(date_key, False):
            objs[chk_obj['_id']] = chk_obj[date_key]


def relavent_data_release_objs(time_ago, orm_obj, exclude_list):
    """Query projects or transactions that has gone past their suspense date."""
    trans_objs = set()
    suspense_args = {
        'suspense_date': 0,
        'suspense_date_0': (
            datetime.now() - time_ago
        ).replace(microsecond=0).isoformat(),
        'suspense_date_1': datetime.now().replace(microsecond=0).isoformat(),
        'suspense_date_operator': 'between'
    }
    suspense_args.update(SearchBase.global_get_args)
    resp = requests.get(
        text_type('{base_url}/{orm_obj}').format(
            base_url=get_config().get('metadata', 'endpoint_url'),
            orm_obj=orm_obj
        ),
        params=suspense_args
    )
    if orm_obj == 'projects':
        for proj_obj in resp.json():
            for rel_type in ['transsip', 'transsap']:
                proj_id = proj_obj['_id']
                if text_type(proj_id) in exclude_list:
                    continue
                resp = requests.get(
                    text_type('{base_url}/{rel_type}?project={proj_id}').format(
                        rel_type=rel_type,
                        base_url=get_config().get('metadata', 'endpoint_url'),
                        proj_id=proj_id
                    )
                )
                for trans_obj in resp.json():
                    trans_objs.add(trans_obj['_id'])
    else:
        for trans_obj in resp.json():
            if text_type(trans_obj['_id']) not in exclude_list:
                trans_objs.add(trans_obj['_id'])
    return trans_objs


def relavent_suspense_date_objs(time_ago, orm_obj, date_key):
    """generate a list of relavent orm_objs saving date_key."""
    objs = {}
    for time_field in ['updated', 'created']:
        obj_args = {
            'time_field': time_field,
            'epoch': (
                datetime.now() - time_ago
            ).replace(microsecond=0).isoformat()
        }
        obj_args.update(SearchBase.global_get_args)
        resp = requests.get(
            text_type('{base_url}/{orm_obj}').format(
                base_url=get_config().get('metadata', 'endpoint_url'),
                orm_obj=orm_obj
            ),
            params=obj_args
        )
        collate_objs_from_key(resp, objs, date_key)
    return objs


def update_suspense_date_objs(objs, time_after, orm_obj):
    """update the list of objs given date_key adding time_after."""
    for obj_id, obj_date_key in objs.items():
        resp = requests.post(
            text_type('{base_url}/{orm_obj}?_id={obj_id}').format(
                base_url=get_config().get('metadata', 'endpoint_url'),
                orm_obj=orm_obj,
                obj_id=obj_id
            ),
            data=dumps(
                {
                    '_id': obj_id,
                    'suspense_date': (
                        parser.parse(obj_date_key) + time_after
                    ).replace(microsecond=0).isoformat()
                }
            ),
            headers={'content-type': 'application/json'}
        )
        assert resp.status_code == 200


def update_data_release(objs):
    """Add objs transactions to the released transactions table."""
    admin_policy = AdminPolicy()
    rel_uuid = admin_policy.get_relationship_info(name='authorized_releaser')[0].get('uuid')
    for trans_id in objs:
        resp = requests.get(
            text_type(
                '{base_url}/transaction_user?transaction={trans_id}&relationship={rel_uuid}'
            ).format(
                base_url=get_config().get('metadata', 'endpoint_url'),
                trans_id=trans_id, rel_uuid=rel_uuid
            )
        )
        if resp.status_code == 200 and resp.json():
            continue
        resp = requests.put(
            text_type(
                '{base_url}/transaction_user'
            ).format(
                base_url=get_config().get('metadata', 'endpoint_url')
            ),
            data=dumps({
                'user': get_config().get('policy', 'admin_user_id'),
                'transaction': trans_id,
                'relationship': rel_uuid
            }),
            headers={'content-type': 'application/json'}
        )
        assert resp.status_code == 200


def data_release(args):
    """
    Data release main subcommand.

    The logic is to query updated objects between now and
    args.time_ago. If the objects args.keyword is set to something
    calculate the suspense date as args.time_after the keyword date.
    Then save the object back to the metadata server.

    The follow on task is to use orm_obj to calculate the released
    data based on the set suspense dates and add that released data
    to the transaction_release table.
    """
    orm_obj, date_key = args.keyword.split('.')
    objs = relavent_suspense_date_objs(args.time_ago, orm_obj, date_key)
    update_suspense_date_objs(objs, args.time_after, orm_obj)
    trans_objs = relavent_data_release_objs(
        args.time_ago, orm_obj, args.exclude)
    update_data_release(trans_objs)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Search transaction rendering methods."""
from six import text_type
from .users import UsersRender
from .base import SearchBase


class InstitutionsRender(SearchBase):
    """Render an insitution for search."""

    fields = [
        'obj_id', 'display_name', 'keyword', 'release',
        'updated_date', 'created_date'
    ]

    @staticmethod
    def obj_id(**inst_obj):
        """Return string for object id."""
        return text_type('institutions_{_id}').format(**inst_obj)

    @staticmethod
    def updated_date(**inst_obj):
        """Return string for the updated date."""
        return text_type('{updated}').format(**inst_obj)

    @staticmethod
    def created_date(**inst_obj):
        """Return string for the created date."""
        return text_type('{created}').format(**inst_obj)

    @staticmethod
    def display_name(**inst_obj):
        """Return the string to render display_name."""
        return text_type('{name}').format(**inst_obj)

    @staticmethod
    def keyword(**inst_obj):
        """Return the rendered string for keywords."""
        return text_type('{name}').format(**inst_obj)

    @classmethod
    def release(cls, **_inst_obj):
        """Return whether the user has released anything."""
        return 'true'

    @classmethod
    def get_transactions(cls, **inst_obj):
        """Return the list of transaction ids for the user."""
        ret = set()
        for inst_user_obj in cls.get_rel_by_args('institution_user', institution=inst_obj['_id']):
            ret.update(UsersRender.get_transactions(_id=inst_user_obj['user']))
        return [
            'transactions_{}'.format(trans_id)
            for trans_id in ret
        ]



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""CherryPy Status Metadata projectinfo base class."""
import requests
from pacifica.policy.config import get_config


# pylint: disable=too-few-public-methods
class QueryBase(object):
    """Formats summary data for other classes down the tree."""

    base_user_info = {
        'emsl_employee': False,
        'project_list': [],
        'instrument_list': []
    }

    @staticmethod
    def _get_user_lookups(url, header_list):
        req = requests.get(
            url=url, headers=header_list
        )
        infolist = [i['id'] for i in req.json() if i]
        return infolist

    @staticmethod
    def get_full_user_info(user_id):
        """Return user information for the given user_id."""
        header_list = {'Content-Type': 'application/json'}
        user_info_request = requests.get(
            url='{0}/userinfo/by_id/{1}'.format(
                get_config().get('metadata', 'endpoint_url'), user_id),
            headers=header_list
        )
        if user_info_request.status_code != 200:
            return None

        user_info = user_info_request.json()

        # get available projects for this user
        proj_list = QueryBase._get_user_lookups(
            '{0}/projectinfo/by_user_id/{1}'.format(
                get_config().get('metadata', 'endpoint_url'),
                user_id
            ),
            header_list
        )
        # get available instruments for this user
        inst_list = QueryBase._get_user_lookups(
            url='{0}/instrumentinfo/by_user_id/{1}'.format(
                get_config().get('metadata', 'endpoint_url'),
                user_id
            ),
            header_list=header_list
        )
        user_info['project_list'] = proj_list
        user_info['instrument_list'] = inst_list

        return user_info

    @staticmethod
    def _merge_two_dicts(dict_a, dict_b):
        """Given two dicts, merge them into a new dict as a shallow copy."""
        dict_out = dict_a.copy()
        if dict_b:
            dict_out.update(dict_b)
        return dict_out



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Test the uploader policy."""
from cherrypy.test import helper
from pacifica.policy.admin import AdminPolicy
from common_test import CommonCPSetup


class TestAdminBase(helper.CPWebCase, CommonCPSetup):
    """Test the uploader policy service."""

    PORT = 8181
    HOST = '127.0.0.1'
    headers = [('Content-Type', 'application/json')]

    def test_ingest_coverage(self):
        """Validate uploader returning xref validation."""
        adm_policy = AdminPolicy()
        valid_pairs = [
            (100, 54),
            (10, 54)
        ]
        # pylint: disable=protected-access
        for user_id, inst_id in valid_pairs:
            for proj_id in adm_policy._projects_for_user_inst(user_id, inst_id):
                self.assertTrue(
                    inst_id in adm_policy._instruments_for_user_proj(user_id, proj_id))
        # pylint: enable=protected-access

    def test_base_queries(self):
        """Test the base class queries."""
        adm_policy = AdminPolicy()
        # pylint: disable=protected-access
        res = adm_policy._projects_for_custodian(10)
        # pylint: enable=protected-access
        self.assertTrue('1234a' in res)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Search transaction rendering methods."""
from six import text_type
from .base import SearchBase


class RelationshipsRender(SearchBase):
    """Render an insitution for search."""

    fields = [
        'obj_id', 'name', 'display_name', 'keyword',
        'updated_date', 'created_date'
    ]

    @staticmethod
    def obj_id(**rel_obj):
        """Return string for object id."""
        return text_type('relationships_{uuid}').format(**rel_obj)

    @staticmethod
    def updated_date(**rel_obj):
        """Return string for the updated date."""
        return text_type('{updated}').format(**rel_obj)

    @staticmethod
    def created_date(**rel_obj):
        """Return string for the created date."""
        return text_type('{created}').format(**rel_obj)

    @staticmethod
    def display_name(**rel_obj):
        """Return the string to render display_name."""
        return text_type('{display_name}').format(**rel_obj)

    @staticmethod
    def keyword(**rel_obj):
        """Return the rendered string for keywords."""
        return text_type('{display_name}').format(**rel_obj)

    @classmethod
    def name(cls, **rel_obj):
        """Return whether the user has released anything."""
        return text_type('{name}').format(**rel_obj)

    @classmethod
    def get_transactions(cls, **_kwargs):
        """Just return an empty list."""
        return []



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Test status policy methods."""
from json import loads, dumps
from cherrypy.test import helper
from common_test import CommonCPSetup


class TestReportingPolicy(helper.CPWebCase, CommonCPSetup):
    """Test the status policy service."""

    PORT = 8181
    HOST = '127.0.0.1'
    headers = [('Content-Type', 'application/json')]

    def test_transaction_details(self):
        """Return transaction details for a list of transactions."""
        transaction_list = [67, 68]
        user_id = 10
        url = '/reporting/transaction_details/{0}'.format(user_id)
        self.getPage(
            url,
            self.headers +
            [('Content-Length', str(len(dumps(transaction_list))))],
            'POST',
            dumps(transaction_list)
        )
        self.assertStatus('200 OK')
        ret_data = loads(self.body.decode('UTF-8'))
        self.assertFalse(ret_data is None)

    def test_transaction_summary(self):
        """Return upload summary results."""
        time_basis = 'modified'
        object_type = 'instrument'
        start_date = '2016-01-01'
        end_date = '2016-12-31'
        object_list = [54]
        user_id = 10
        url = '/reporting/transaction_summary/{0}/{1}/{2}/{3}?user={4}'.format(
            time_basis, object_type, start_date, end_date, user_id
        )
        self.getPage(
            url,
            self.headers + [('Content-Length', str(len(dumps(object_list))))],
            'POST',
            dumps(object_list)
        )
        self.assertStatus('200 OK')
        self.assertFalse(self.body is None)

        user_id = 150
        url = '/reporting/transaction_summary/{0}/{1}/{2}/{3}?user={4}'.format(
            time_basis, object_type, start_date, end_date, user_id
        )
        self.getPage(
            url,
            self.headers + [('Content-Length', str(len(dumps(object_list))))],
            'POST',
            dumps(object_list)
        )
        self.assertStatus('200 OK')
        # self.assertStatus('200 OK')
        # self.assertTrue(loads(self.body) is None)

    def test_bad_transaction_details(self):
        """Return transaction details for a list of transactions."""
        transaction_list = [67, 68]
        user_id = 150
        url = '/reporting/transaction_details/{0}'.format(user_id)
        self.getPage(
            url,
            self.headers +
            [('Content-Length', str(len(dumps(transaction_list))))],
            'POST',
            dumps(transaction_list)
        )
        self.assertStatus('200 OK')
        self.assertFalse(loads(self.body.decode('UTF-8')))



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Search transaction rendering methods."""
from six import text_type
from .keys import KeysRender
from .values import ValuesRender
from .base import SearchBase


class InstrumentsRender(SearchBase):
    """Render an insitution for search."""

    fields = [
        'obj_id', 'display_name', 'keyword', 'release',
        'updated_date', 'created_date'
    ]
    rel_objs = ['key_value_pairs']

    @staticmethod
    def obj_id(**instrument_obj):
        """Return string for object id."""
        return text_type('instruments_{_id}').format(**instrument_obj)

    @staticmethod
    def updated_date(**instrument_obj):
        """Return string for the updated date."""
        return text_type('{updated}').format(**instrument_obj)

    @staticmethod
    def created_date(**instrument_obj):
        """Return string for the created date."""
        return text_type('{created}').format(**instrument_obj)

    @staticmethod
    def display_name(**instrument_obj):
        """Return the string to render display_name."""
        return text_type('{display_name}').format(**instrument_obj)

    @staticmethod
    def keyword(**instrument_obj):
        """Return the rendered string for keywords."""
        return text_type('{name}').format(**instrument_obj)

    @classmethod
    def release(cls, **_instrument_obj):
        """Return whether the user has released anything."""
        return 'true'

    @classmethod
    def key_value_pairs_obj_lists(cls, **inst_obj):
        """Get the key value pairs related to the transaction."""
        ret = {
            'key_value_hash': {},
            'key_objs': [],
            'value_objs': []
        }
        inst_kvp_objs = cls.get_rel_by_args(
            'instrument_key_value',
            instrument=inst_obj['_id'],
            relationship=cls.search_required_uuid
        )
        for inst_kvp_obj in inst_kvp_objs:
            key_obj = cls.get_rel_by_args('keys', _id=inst_kvp_obj['key'])[0]
            value_obj = cls.get_rel_by_args('values', _id=inst_kvp_obj['value'])[0]
            ret['key_objs'].append(KeysRender.render(key_obj))
            ret['value_objs'].append(ValuesRender.render(value_obj))
            ret['key_value_hash'][key_obj['key']] = value_obj['value']
        return ret

    # this doesn't get executed but is needed to be here to satisfy abstract method
    @classmethod
    def get_transactions(cls, **inst_obj):
        """Return the list of transaction ids for the user."""
        return [
            'transactions_{}'.format(transsip_obj['_id'])
            for transsip_obj in cls.get_rel_by_args('transsip', instrument=inst_obj['_id'])
        ]



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Test the uploader with httpretty."""
from unittest import TestCase
from json import dumps
import httpretty
from pacifica.policy.uploader.rest import UploaderPolicy
from pacifica.policy.config import get_config


class TestUploader(TestCase):
    """Test the uploader policy with httpretty."""

    sample_user_id = 23
    admin_user_id = 45
    admin_group_id = 127
    user_group_json = [
        {
            'group': admin_group_id,
            'person': admin_user_id
        }
    ]
    admin_group_json = [
        {
            '_id': admin_group_id
        }
    ]
    group_url = '{0}/groups'.format(
        get_config().get('metadata', 'endpoint_url')
    )
    user_group_url = '{0}/user_group'.format(
        get_config().get('metadata', 'endpoint_url')
    )

    @httpretty.activate
    def test_failed_admin_id(self):
        """check failed admin id fallback works."""
        httpretty.register_uri(httpretty.GET, self.group_url,
                               body=dumps([]),
                               content_type='application/json')
        httpretty.register_uri(httpretty.GET, self.user_group_url,
                               body=dumps([]),
                               content_type='application/json')
        upolicy = UploaderPolicy()
        # pylint: disable=no-member
        # pylint: disable=protected-access
        ret = upolicy._is_admin(10)
        # pylint: enable=protected-access
        # pylint: enable=no-member
        self.assertFalse(ret)



#!/usr/bin/python
# -*- coding: utf-8 -*-
# pylint: disable=invalid-name
"""This is the main policy server script."""
import cherrypy
from pacifica.policy.root import Root, error_page_default
from pacifica.policy.globals import CHERRYPY_CONFIG


cherrypy.config.update({'error_page.default': error_page_default})
# pylint doesn't realize that application is actually a callable
# pylint: disable=invalid-name
application = cherrypy.Application(Root(), '/', CHERRYPY_CONFIG)
# pylint: enable=invalid-name



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
The CherryPy rest object for the structure.

Below is an example post body::

    [
        {"destinationTable": "Transactions._id", "value": 1234},
        {"destinationTable": "Transactions.submitter", "value": 34002},
        {"destinationTable": "Transactions.project", "value": "34002"},
        {"destinationTable": "Transactions.instrument", "value": 34002},
        {"destinationTable": "TransactionKeyValue", "key": "Tag", "value": "Blah"},
        {"destinationTable": "TransactionKeyValue", "key": "Taggy", "value": "Blah"},
        {"destinationTable": "TransactionKeyValue", "key": "Taggier", "value": "Blah"}
        {
            "destinationTable": "Files",
            "_id": 34, "name": "foo.txt", "subdir": "a/b/",
            "ctime": "Tue Nov 29 14:09:05 PST 2016",
            "mtime": "Tue Nov 29 14:09:05 PST 2016",
            "size": 128, "mimetype": "text/plain"
        },
        {
            "destinationTable": "Files",
            "_id": 35, "name": "bar.txt", "subdir": "a/b/",
            "ctime": "Tue Nov 29 14:09:05 PST 2016",
            "mtime": "Tue Nov 29 14:09:05 PST 2016",
            "size": 47, "mimetype": "text/plain"
        },
    ]
"""
from six import text_type
from cherrypy import tools, request, HTTPError
from pacifica.policy.uploader.rest import UploaderPolicy


# pylint: disable=too-few-public-methods
class IngestPolicy(UploaderPolicy):
    """CherryPy Ingest Policy Class."""

    @staticmethod
    def _pull_data_by_rec(query, table):
        """Pull the value for the table."""
        for rec in query:
            if rec['destinationTable'] == table:
                return rec['value']
        return None

    def _valid_query(self, query):
        """Validate the metadata format."""
        variables_to_query = {
            'submitter': 'users',
            'project': 'projects',
            'instrument': 'instruments'
        }
        invalid_terms = []
        valid_terms = {}
        for variable in variables_to_query:
            value = self._pull_data_by_rec(
                query, 'Transactions.{0}'.format(variable))
            valid = self._object_id_valid(variables_to_query[variable], value)
            if not valid:
                invalid_terms.append(variable)
            else:
                valid_terms[variable] = value
        if not invalid_terms:
            # all the incoming terms are valid, check for xrefs
            if text_type(valid_terms['project']) not in self._projects_for_user_inst(
                    valid_terms['submitter'], valid_terms['instrument']):
                invalid_terms.append(
                    text_type('project ({}) not in user instrument list ({})').format(
                        text_type(valid_terms['project']),
                        self._projects_for_user_inst(
                            valid_terms['submitter'],
                            valid_terms['instrument']
                        )
                    )
                )
            if int(valid_terms['instrument']) not in self._instruments_for_user_proj(
                    valid_terms['submitter'], valid_terms['project']
            ):
                invalid_terms.append(
                    text_type('instrument ({}) not in user project list ({})').format(
                        int(valid_terms['instrument']),
                        self._instruments_for_user_proj(
                            valid_terms['submitter'],
                            valid_terms['project']
                        )
                    )
                )
            if not invalid_terms:
                return {'status': 'success'}

        raise HTTPError(412, text_type(
            'Precondition Failed: Invalid values for {0}').format(', '.join(invalid_terms)))

    # pylint: disable=invalid-name
    @tools.json_in()
    @tools.json_out()
    def POST(self):
        """Read in the json query and return results."""
        metadata = request.json
        return self._valid_query(metadata)
    # pylint: enable=invalid-name
# pylint: enable=too-few-public-methods



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""The CherryPy rest object for the structure."""
from json import dumps
from cherrypy import tools, request, HTTPError
from pacifica.policy.admin import AdminPolicy


# pylint: disable=too-few-public-methods
class UploaderPolicy(AdminPolicy):
    """
    CherryPy root object class.

    not exposed by default the base objects are exposed
    """

    exposed = True

    @staticmethod
    def _filter_results(results, *args):
        for result in results:
            for key in list(result.keys()):
                if key not in args:
                    del result[key]

    @staticmethod
    def _clean_user_query_id(query):
        """determine the user_id for whatever is in the query."""
        try:
            return int(query['user'])
        except ValueError:
            return None

    def _user_info_from_queries(self, user_queries):
        ret = []
        for user_query in user_queries:
            ret.extend(self._user_info_from_kwds(**user_query))
        return ret

    def _query_select_user_info(self, query):
        user_id = self._clean_user_query_id(query)
        where_objects = query['where'].keys()
        user_queries = []
        if 'network_id' in where_objects:
            user_queries.append({'network_id': query['where']['network_id']})
        elif 'project_id' in where_objects:
            for proj_user_id in self._users_for_proj(query['where']['project_id']):
                user_queries.append({'_id': proj_user_id})
        else:
            user_queries.append({'_id': user_id})
        return self._user_info_from_queries(user_queries)

    def _query_select_project_info(self, query):
        user_id = self._clean_user_query_id(query)
        where_objects = query['where'].keys()
        if 'instrument_id' in where_objects:
            proj_ids = self._projects_for_user_inst(
                user_id, query['where']['instrument_id'])
        elif '_id' in query['where']:
            proj_ids = [query['where']['_id']]
        else:
            proj_ids = self._projects_for_user(user_id)
        return self._project_info_from_ids(proj_ids)

    def _query_select_instrument_info(self, query):
        user_id = self._clean_user_query_id(query)
        if 'project_id' in query['where']:
            if self._is_admin(user_id):
                return self._all_instrument_info()
            inst_ids = self._instruments_for_user_proj(
                user_id, query['where']['project_id'])
        elif '_id' in query['where']:
            inst_ids = [query['where']['_id']]
        else:
            inst_ids = self._instruments_for_user(user_id)
        return self._instrument_info_from_ids(inst_ids)

    def _query_select_admin(self, query):
        wants_object = query['from']
        if wants_object == 'users':
            return self._query_select_user_info(query)
        if wants_object == 'projects':
            if '_id' in query['where']:
                proj_ids = [query['where']['_id']]
                return self._project_info_from_ids(proj_ids)
            return self._all_project_info()
        if wants_object == 'instruments':
            return self._query_select_instrument_info(query)
        raise TypeError('Invalid Query: ' +
                        'Not sure how to want {0} where {1}'.format(wants_object, query['where']))

    def _query_select(self, query):
        wants_object = query['from']
        if wants_object == 'users':
            return self._query_select_user_info(query)
        if wants_object == 'projects':
            return self._query_select_project_info(query)
        if wants_object == 'instruments':
            return self._query_select_instrument_info(query)
        raise TypeError('Invalid Query: ' +
                        'Not sure how to want {0} where {1}'.format(wants_object, query['where']))

    @staticmethod
    def _valid_query(query):
        if 'user' not in query:
            return False
        if 'from' not in query:
            return False
        if 'columns' not in query:
            return False
        return True

    # pylint: disable=invalid-name
    @tools.json_out()
    @tools.json_in()
    def POST(self):
        """Read in the json query and return results."""
        query = request.json
        if not self._valid_query(query):
            raise HTTPError(500, dumps(
                {'message': 'Invalid Query.', 'status': 'error'}))
        if self._is_admin(self._clean_user_query_id(query)):
            results = self._query_select_admin(query)
        else:
            results = self._query_select(query)
        self._filter_results(results, *(query['columns']))
        return results
    # pylint: enable=invalid-name
# pylint: enable=too-few-public-methods



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""CherryPy Status Metadata object class."""
from six import text_type
import requests
from cherrypy import tools, request
from pacifica.policy.validation import validate_user
from pacifica.policy.config import get_config
from pacifica.policy.reporting.transaction.query_base import QueryBase


# pylint: disable=too-few-public-methods
class TransactionSummary(QueryBase):
    """Retrieves a summary of all transactions matching the search criteria."""

    exposed = True

    @staticmethod
    # pylint: disable=too-many-arguments
    def _get_transaction_list_summary(
            time_basis, object_list, object_type, start_date, end_date, user_id
    ):
        header_list = {'Content-Type': 'application/json'}
        url = '{0}/summaryinfo/by_date/'.format(
            get_config().get('metadata', 'endpoint_url')
        )
        url += '{0}/{1}/{2}/{3}'.format(
            time_basis, object_type, start_date, end_date
        )
        req = requests.post(
            url=url,
            json=object_list,
            headers=header_list
        )
        user_info = QueryBase._merge_two_dicts(
            QueryBase.base_user_info,
            QueryBase.get_full_user_info(user_id),
        )

        req_json = req.json()

        types_to_check = ['instrument', 'project']

        for entry_type in types_to_check:
            req_json['summary_totals']['upload_stats'][entry_type] = TransactionSummary._cleanup_object_stats(
                object_listing=req_json['summary_totals']['upload_stats'][entry_type],
                object_type=entry_type,
                user_info=user_info
            )

        return req_json

    @staticmethod
    def _cleanup_object_stats(object_listing, object_type, user_info):
        valid_object_list = map(text_type, user_info[object_type + '_list'])
        clean_object_stats = {}
        for object_id, object_stats in object_listing.items():
            if object_id in valid_object_list or user_info['emsl_employee']:
                clean_object_stats[object_id] = object_stats
            else:
                if 'Other' not in clean_object_stats.keys():
                    clean_object_stats['Other'] = 0
                clean_object_stats['Other'] += object_stats
        return clean_object_stats

    # pylint: enable=too-many-arguments

    # CherryPy requires these named methods
    # pylint: disable=invalid-name
    @staticmethod
    @tools.json_in()
    @tools.json_out()
    @validate_user('user')
    def POST(time_basis=None, object_type=None, start_date=None, end_date=None, **kwargs):
        """CherryPy GET method."""
        object_list = request.json
        user_id = kwargs['user']
        return TransactionSummary._get_transaction_list_summary(
            time_basis, object_list, object_type, start_date, end_date, user_id
        )
# pylint: enable=too-few-public-methods



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Base class module for standard queries for the upload status tool."""
from pacifica.policy.config import get_config
from pacifica.policy.admin import AdminPolicy


# pylint: disable=too-few-public-methods
class QueryBase(AdminPolicy):
    """This pulls the common bits of instrument and project query into a single class."""

    md_url = get_config().get('metadata', 'endpoint_url')
    all_instruments_url = '{0}/instruments'.format(md_url)
    all_projects_url = '{0}/projects'.format(md_url)
    all_transactions_url = '{0}/transactions'.format(md_url)

    def _get_available_projects(self, user_id):
        return self._projects_for_user(user_id)
# pylint: enable=too-few-public-methods



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Search transaction rendering methods."""
from six import text_type
from .base import SearchBase


class ScienceThemesRender(SearchBase):
    """Render an insitution for search."""

    fields = [
        'obj_id', 'display_name', 'keyword', 'release',
        'updated_date', 'created_date'
    ]

    @staticmethod
    def obj_id(**proj_obj):
        """Return string for object id."""
        return text_type('science_themes_{science_theme}').format(**proj_obj)

    @staticmethod
    def updated_date(**proj_obj):
        """Return string for the updated date."""
        return text_type('{updated}').format(**proj_obj)

    @staticmethod
    def created_date(**proj_obj):
        """Return string for the created date."""
        return text_type('{created}').format(**proj_obj)

    @staticmethod
    def display_name(**proj_obj):
        """Return the string to render display_name."""
        return text_type('{science_theme}').format(**proj_obj)

    @staticmethod
    def keyword(**proj_obj):
        """Return the rendered string for keywords."""
        return text_type('{science_theme}').format(**proj_obj)

    @classmethod
    def release(cls, **_proj_obj):
        """Return whether the user has released anything."""
        return 'true'

    @classmethod
    def get_transactions(cls, **proj_obj):
        """Return the list of transaction ids for the user."""
        ret = set()
        for rel_proj_obj in cls.get_rel_by_args('projects', science_theme=proj_obj['science_theme']):
            ret.update([
                'transactions_{}'.format(trans_id)
                for trans_id in cls._transsip_transsap_merge({'project': rel_proj_obj['_id']}, '_id')
            ])
        return list(ret)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""The Admin module has logic about checking for admin group info."""
from __future__ import absolute_import
from json import loads
import requests
from .config import get_config

RECURSION_DEPTH = 0


# pylint: disable=too-few-public-methods
class AdminPolicy(object):
    """
    Enforces the admin policy.

    Base class for checking for admin group membership or not.
    """

    md_url = get_config().get('metadata', 'endpoint_url')
    all_users_url = '{0}/users'.format(md_url)
    all_instruments_url = '{0}/instruments'.format(md_url)
    all_projects_url = '{0}/projects'.format(md_url)
    all_relationships_url = '{0}/relationships'.format(md_url)
    proj_user_url = '{0}/project_user'.format(md_url)
    proj_instrument_url = '{0}/project_instrument'.format(md_url)
    inst_user_url = '{0}/instrument_user'.format(md_url)
    inst_group_url = '{0}/instrument_group'.format(md_url)

    def __init__(self):
        """Constructor for Uploader Policy."""
        self.admin_group = get_config().get('policy', 'admin_group')
        self.admin_group_id = get_config().get('policy', 'admin_group_id')

    def _format_url(self, url, **get_args):
        """Append the recursion_depth parameter to the url."""
        get_args['recursion_depth'] = RECURSION_DEPTH
        args_str = '&'.join(
            [u'{0}={1}'.format(key, value) for key, value in get_args.items()]
        )
        return u'{0}?{1}'.format(getattr(self, url), args_str)

    def get_relationship_info(self, **get_args):
        """Get a relationship by kwargs."""
        return loads(requests.get(self._format_url('all_relationships_url', **get_args)).text)

    def _all_project_info(self):
        return loads(requests.get(self._format_url('all_projects_url')).text)

    def _all_instrument_info(self):
        return loads(requests.get(self._format_url('all_instruments_url')).text)

    def _projects_for_user(self, user_id, relationship='member_of'):
        if self._is_admin(user_id):
            return [proj['_id'] for proj in self._all_project_info()]
        rel_uuid = self.get_relationship_info(name=relationship)[0].get('uuid')
        proj_url = self._format_url('proj_user_url', user=user_id, relationship=rel_uuid)
        return [part['project'] for part in loads(requests.get(proj_url).text)]

    def _projects_for_custodian(self, user_id):
        inst_list = self._instruments_for_custodian(user_id)
        projects_for_custodian = set([])
        for inst in inst_list:
            projects = self._projects_for_inst(inst)
            projects_for_custodian = projects_for_custodian.union(projects)
        return list(projects_for_custodian)

    def _instruments_for_custodian(self, user_id):
        rel_uuid = self.get_relationship_info(name='custodian')[0].get('uuid')
        inst_custodian_associations_url = self._format_url(
            'inst_user_url', user=user_id, relationship=rel_uuid)
        inst_custodian_list = loads(requests.get(
            inst_custodian_associations_url).text)
        return [i['instrument'] for i in inst_custodian_list]

    def _projects_for_inst(self, inst_id):
        inst_projs_url = self._format_url(
            'proj_instrument_url',
            instrument_id=inst_id
        )
        inst_projs = loads(requests.get(inst_projs_url).text)
        inst_projs = set([part['project'] for part in inst_projs])
        return inst_projs

    def _projects_for_user_inst(self, user_id, inst_id):
        projs = set(self._projects_for_user(user_id))
        projs_for_custodian = set(self._projects_for_custodian(user_id))
        inst_groups = self._groups_for_inst(inst_id)
        if inst_groups:
            group_insts = set()
            for group_id in inst_groups:
                group_insts |= set(self._instruments_for_group(group_id))
        else:
            group_insts = set([inst_id])
        ginst_projs = set()
        for ginst_id in group_insts:
            ginst_projs |= self._projects_for_inst(ginst_id)
        return list((projs | projs_for_custodian) & ginst_projs)

    def _project_info_from_ids(self, proj_list):
        ret = []
        if proj_list:
            for proj_id in proj_list:
                proj_url = self._format_url('all_projects_url', _id=proj_id)
                ret.extend(loads(requests.get(proj_url).text))
        return ret

    def _groups_for_inst(self, inst_id):
        inst_g_url = self._format_url('inst_group_url', instrument_id=inst_id)
        return [i['group'] for i in loads(requests.get(inst_g_url).text)]

    def _instruments_for_group(self, group_id):
        inst_g_url = self._format_url('inst_group_url', group_id=group_id)
        return [i['instrument'] for i in loads(requests.get(inst_g_url).text)]

    def _instruments_for_user(self, user_id):
        if self._is_admin(user_id):
            return [inst['_id'] for inst in self._all_instrument_info()]
        return self._instruments_for_custodian(user_id)

    def _instruments_for_user_proj(self, user_id, proj_id):
        user_insts = set(self._instruments_for_user(user_id))
        if self._is_admin(user_id):
            return list(user_insts)
        proj_insts_url = self._format_url(
            'proj_instrument_url', project=proj_id)
        proj_insts = set([part['instrument']
                          for part in loads(requests.get(proj_insts_url).text)])
        inst_groups = set()
        for inst_id in proj_insts:
            inst_groups |= set(self._groups_for_inst(inst_id))
        group_insts = set()
        for group_id in inst_groups:
            group_insts |= set(self._instruments_for_group(group_id))
        return list(group_insts | user_insts | proj_insts)

    def _instrument_info_from_ids(self, inst_list):
        ret = []
        for inst_id in inst_list:
            inst_url = self._format_url('all_instruments_url', _id=inst_id)
            ret.extend(loads(requests.get(inst_url).text))
        return ret

    def _users_for_proj(self, proj_id):
        user_proj_url = self._format_url(
            'proj_user_url', project=proj_id)
        user_projs = loads(requests.get(user_proj_url).text)
        return list(set([str(part['user']) for part in user_projs]))

    def _user_info_from_kwds(self, **kwds):
        return loads(requests.get(self._format_url('all_users_url', **kwds)).text)

    @staticmethod
    def _object_id_valid(object_lookup_name, object_id):
        if not object_id:
            return False
        object_type_url = '{0}/{1}'.format(get_config().get('metadata', 'endpoint_url'),
                                           object_lookup_name)
        object_query_url = u'{0}?_id={1}'.format(object_type_url, object_id)
        object_value_req = requests.get(object_query_url)
        object_is_valid = loads(object_value_req.text)
        return len(object_is_valid) > 0

    def _is_admin(self, user_id):
        amember_query = '{0}/user_group?group={1}&person={2}'.format(
            self.md_url,
            self.admin_group_id,
            user_id
        )
        is_admin_req = requests.get(amember_query)
        is_admin_list = loads(is_admin_req.text)
        return len(is_admin_list) > 0
# pylint: enable=too-few-public-methods



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Test status policy methods."""
from json import loads, dumps
from cherrypy.test import helper
from common_test import CommonCPSetup


class TestStatusPolicy(helper.CPWebCase, CommonCPSetup):
    """Test the status policy service."""

    PORT = 8181
    HOST = '127.0.0.1'
    headers = [('Content-Type', 'application/json')]

    def test_instrument_by_project(self):
        """Return instruments for a specified project."""
        url = '/status/instrument/by_project_id/1234a'
        self.getPage(url)
        self.assertStatus('200 OK')
        answer = loads(self.body.decode('UTF-8'))
        self.assertEqual(len(answer.get('items')), 2)

        # Test nonexistent project
        url = '/status/instrument/by_project_id/3455'
        self.getPage(url)
        self.assertStatus('404 Not Found')

        # Test bad project format
        url = '/status/instrument/by_project_id/bob123'
        self.getPage(url)
        self.assertStatus('400 Invalid Request')

        # Test with no instruments to return
        url = '/status/instrument/by_project_id/1238'
        self.getPage(url)
        self.assertStatus('200 OK')
        answer = loads(self.body.decode('UTF-8'))
        self.assertEqual(len(answer.get('items')), 0)

    def test_files_for_transaction(self):
        """Return file listing for a transaction."""
        transaction_id = 67
        url = '/status/transactions/files/{0}'.format(transaction_id)
        self.getPage(url)
        self.assertStatus('200 OK')
        answer = loads(self.body.decode('UTF-8'))
        self.assertEqual(len(answer), 2)
        self.assertListEqual(sorted(answer.keys()),
                             sorted(loads(dumps(['103', '104']))))

    def test_transaction_for_id(self):
        """Return transaction info for a given id."""
        transaction_id = 68
        url = '/status/transactions/by_id/{0}'.format(transaction_id)
        self.getPage(url)
        self.assertStatus('200 OK')
        answer = loads(self.body.decode('UTF-8'))
        self.assertEqual(answer['_id'], 68)

        bad_transaction_id = 'bob'
        url = '/status/transactions/by_id/{0}'.format(bad_transaction_id)
        self.getPage(url)
        self.assertStatus('400 Invalid Request')
        self.assertInBody('Provide an appropriate transaction_id')

    def test_transaction_search(self):
        """Return projects for a specified user."""
        transaction_id = 68
        url = '/status/transactions/search?transaction={0}'.format(
            transaction_id)
        self.getPage(url)
        self.assertStatus('200 OK')

    def test_user_query(self):
        """Test user details retrieval."""
        user_id = 10
        url = '/status/users/by_id/{0}'.format(user_id)
        self.getPage(url)
        self.assertStatus('200 OK')

    def test_user_search(self):
        """Test user search functionality."""
        search_term = 'dmlb2001'
        url = '/status/users/search/{0}/simple'.format(search_term)
        self.getPage(url)
        self.assertStatus('200 OK')
        self.assertInBody('Brown')

        search_term = 'dmlb2004'
        url = '/status/users/search/{0}/simple'.format(search_term)
        self.getPage(url)
        self.assertStatus(404)

    def test_projects_by_user(self):
        """Return projects for a specified user."""
        user_id = 10
        url = '/status/projects/by_user_id/{0}'.format(user_id)
        self.getPage(url)
        self.assertStatus('200 OK')
        answer = loads(self.body.decode('UTF-8'))
        self.assertEqual(len(answer), 2)

        user_id = 13
        url = '/status/projects/by_user_id/{0}'.format(user_id)
        self.getPage(url)
        self.assertStatus('200 OK')
        self.assertInBody('404 User Does Not Exist')

    def test_project_lookup(self):
        """Return an appropriate project for a specified project_id."""
        project_id = '1234a'
        url = '/status/projects/by_project_id/{0}'.format(project_id)
        self.getPage(url)
        self.assertStatus('200 OK')
        answer = loads(self.body.decode('UTF-8'))
        self.assertEqual(len(answer['instruments']), 1)
        self.assertEqual(answer['id'], project_id)

    def test_instrument_search(self):
        """Return appropriate instruments for a set of search criteria."""
        search_terms = 'nmr'
        user_id = 10
        url = '/status/instrument/search/{0}?user={1}'.format(
            search_terms, user_id)
        self.getPage(url)
        self.assertStatus('200 OK')

        user_id = 12
        url = '/status/instrument/search/{0}?user={1}'.format(
            search_terms, user_id)
        self.getPage(url)
        self.assertStatus('200 OK')
        self.assertBody('[]')

        user_id = 100
        url = '/status/instrument/search/{0}?user={1}'.format(
            search_terms, user_id)
        self.getPage(url)
        self.assertStatus('200 OK')

        search_terms = 'flux+capacitor'
        url = '/status/instrument/search/{0}?user={1}'.format(
            search_terms, user_id)
        self.getPage(url)
        self.assertStatus(404)

    def test_project_search(self):
        """Return appropriate projects for a search term and user id."""
        search_term = 'expired'
        user_id = 10  # admin user
        url = '/status/projects/search/{0}?user={1}'.format(
            search_term, user_id)
        self.getPage(url)
        self.assertStatus('200 OK')
        answer = loads(self.body.decode('UTF-8'))
        self.assertEqual(len(answer), 2)

        search_term = None
        url = '/status/projects/search?user={0}'.format(user_id)
        self.getPage(url)
        self.assertStatus('200 OK')
        answer = loads(self.body.decode('UTF-8'))
        self.assertEqual(len(answer), 2)

        search_term = 'pacifica'
        user_id = 100  # non admin user
        url = '/status/projects/search/{0}?user={1}'.format(
            search_term, user_id)
        self.getPage(url)
        self.assertStatus('200 OK')
        answer = loads(self.body.decode('UTF-8'))
        self.assertEqual(len(answer), 1)

        search_term = 'bob'
        url = '/status/projects/search/{0}'.format(search_term)
        self.getPage(url)
        self.assertStatus('200 OK')
        answer = loads(self.body.decode('UTF-8'))
        self.assertEqual(len(answer), 0)

        search_term = 'bob'
        url = '/status/projects/search/{0}?user={1}'.format(
            search_term, user_id)
        self.getPage(url)
        self.assertStatus('200 OK')
        answer = loads(self.body.decode('UTF-8'))
        self.assertEqual(len(answer), 0)

        url = '/status/projects/search?user={0}'.format(user_id)
        self.getPage(url)
        self.assertStatus('200 OK')
        answer = loads(self.body.decode('UTF-8'))
        self.assertEqual(len(answer), 1)

        user_id = 10  # admin user
        search_term = 'nmr'
        url = '/status/instrument/search/{0}?user={1}'.format(
            search_term, user_id)
        self.getPage(url)
        self.assertStatus('200 OK')



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""This is the render object for the search interface."""
import importlib
from six import text_type
from .config import get_config

ELASTIC_INDEX = get_config().get('elasticsearch', 'index')


class SearchRender(object):
    """Search render class to contain methods."""

    @staticmethod
    def get_render_class(obj_cls):
        """Get the render class dynamically."""
        obj_mod = importlib.import_module('.search.{}'.format(obj_cls), 'pacifica.policy')
        parts = [obj_cls_part.capitalize() for obj_cls_part in obj_cls.split('_')]
        return getattr(obj_mod, '{}Render'.format(''.join(parts)))

    @classmethod
    def generate(cls, obj_cls, objs):
        """generate the institution object."""
        render_cls = cls.get_render_class(obj_cls)
        for obj in objs:
            yield {
                '_op_type': 'update',
                '_index': ELASTIC_INDEX,
                '_type': 'doc',
                '_id': render_cls.obj_id(**obj),
                'doc': render_cls.render(obj, True, obj_cls != 'transactions'),
                'doc_as_upsert': True
            }
            if obj_cls == 'projects':
                st_render_cls = cls.get_render_class('science_themes')
                yield {
                    '_op_type': 'update',
                    '_index': ELASTIC_INDEX,
                    '_type': 'doc',
                    '_id': text_type('science_themes_{}').format(obj['science_theme']),
                    'doc': st_render_cls.render(obj, True, True),
                    'doc_as_upsert': True
                }



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Common server setup code for CherryPy testing."""
import logging
from json import dumps, loads
import cherrypy
from pacifica.policy.root import Root, error_page_default
from pacifica.policy.globals import CHERRYPY_CONFIG


class CommonCPSetup(object):
    """Common CherryPy setup class."""

    # pylint: disable=no-member
    def get_json_page(self, path, valid_query):
        """Get a json page and validate its return format."""
        self.getPage(path,
                     self.headers +
                     [('Content-Length', str(len(dumps(valid_query))))],
                     'POST',
                     dumps(valid_query))
        self.assertStatus('200 OK')
        self.assertHeader('Content-Type', 'application/json')
        return loads(self.body.decode('UTF-8'))

    @staticmethod
    def setup_server():
        """Setup each test by starting the CherryPy server."""
        logger = logging.getLogger('urllib2')
        logger.setLevel(logging.DEBUG)
        logger.addHandler(logging.StreamHandler())
        cherrypy.config.update({'error_page.default': error_page_default})
        cherrypy.config.update(CHERRYPY_CONFIG)
        cherrypy.tree.mount(Root(), '/', CHERRYPY_CONFIG)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""The CherryPy rest object for the structure."""
# from pacifica.policy.uploader.instrument import InstrumentQuery
from pacifica.policy.status.project_query import ProjectQuery
from pacifica.policy.status.instrument_query import InstrumentQuery
from pacifica.policy.status.transaction_query import TransactionQuery
from pacifica.policy.status.user_query import UserQuery


# pylint: disable=too-few-public-methods
class StatusPolicy(object):
    """
    CherryPy root object class.

    not exposed by default the base objects are exposed.

    """

    exposed = False

    def __init__(self):
        """Create local objects to allow for import to work."""
        self.instruments = InstrumentQuery()
        self.instrument = InstrumentQuery()
        self.projects = ProjectQuery()
        self.transactions = TransactionQuery()
        self.users = UserQuery()
# pylint: enable=too-few-public-methods



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Test the admin command line."""
from unittest import TestCase
import requests
from jsonschema import validate
from pacifica.policy.admin_cmd import main


class TestAdminCMD(TestCase):
    """Test the admin command line tools."""

    project_abstract = u'This is a currently active project with a set end date but no clos\u00e9d date'
    es_schema = {
        'type': 'object',
        'properties': {
            '_source': {
                'type': 'object',
                'properties': {
                    'type': {'const': 'transactions'},
                    'obj_id': {'const': 'transactions_67'},
                    'access_url': {'const': 'https://dx.doi.org/10.25584/data.2018-03.127/123456'},
                    'has_doi': {'const': 'true'},
                    'release': {'const': 'true'},
                    'created_date': {'const': '2017-07-15T00:00:00'},
                    'users': {
                        'type': 'object',
                        'properties': {
                            'submitter': {
                                'type': 'array',
                                'items': {
                                    'type': 'object',
                                    'properties': {
                                        'type': {'const': 'users'},
                                        'obj_id': {'const': 'users_10'},
                                        'display_name': {'const': u'Brown\u00e9 Jr, David\u00e9 '},
                                        'keyword': {'const': u'Brown\u00e9 Jr, David\u00e9 '}
                                    }
                                }
                            }
                        }
                    },
                    'institutions': {
                        'type': 'array',
                        'items': {
                            'type': 'object',
                            'properties': {
                                'type': {'const': 'institutions'},
                                'obj_id': {'const': 'institutions_47'},
                                'display_name': {'const': u'University of Washington\u00e9'},
                                'keyword': {'const': u'University of Washington\u00e9'}
                            }
                        }
                    },
                    'instruments': {
                        'type': 'array',
                        'items': {
                            'type': 'object',
                            'properties': {
                                'type': {'const': 'instruments'},
                                'obj_id': {'const': 'instruments_54'},
                                'display_name': {'const': u'NMR PROBES: Nittany Liquid Prob\u00e9s'},
                                'long_name': {'const': u'NMR PROBES: Nittany Liquid Prob\u00e9s'},
                                'keyword': {'const': u'NMR PROBES: Nittany Liquid Prob\u00e9s'},
                                'key_value_pairs': {
                                    'type': 'object',
                                    'properties': {
                                        'key_value_hash': {
                                            'type': 'object',
                                            'properties': {
                                                'temp_f': {'const': '19'},
                                            }
                                        },
                                        'key_value_objs': {
                                            'type': 'array',
                                            'items': {
                                                'type': 'object',
                                                'properties': {
                                                    'key': {'const': 'temp_f'},
                                                    'value': {'const': '19'}
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    },
                    'instrument_groups': {
                        'type': 'array',
                        'items': {
                            'type': 'object',
                            'properties': {
                                'type': {'const': 'groups'},
                                'obj_id': {'const': 'groups_1001'},
                                'display_name': {'const': u'nmr_instrum\u00e9nts'},
                                'keyword': {'const': u'nmr_instrum\u00e9nts'}
                            }
                        }
                    },
                    'projects': {
                        'type': 'array',
                        'items': {
                            'type': 'object',
                            'properties': {
                                'type': {'const': 'projects'},
                                'obj_id': {'const': 'projects_1234a'},
                                'display_name': {'const': u'Pacifica D\u00e9velopment (active no close)'},
                                'long_name': {'const': ''},
                                'abstract': {'const': project_abstract},
                                'title': {'const': u'Pacifica D\u00e9velopment (active no close)'},
                                'keyword': {'const': u'Pacifica D\u00e9velopment (active no close)'},
                            }
                        }
                    },
                    'science_themes': {
                        'type': 'array',
                        'items': {
                            'type': 'object',
                            'properties': {
                                'type': {'const': 'science_themes'},
                                'obj_id': {'const': u'science_themes_g\u00e9neral'},
                                'display_name': {'const': u'g\u00e9neral'}
                            }
                        }
                    },
                    'key_value_pairs': {
                        'type': 'object',
                        'properties': {
                            'key_value_hash': {
                                'type': 'object',
                                'properties': {
                                    'temp_f': {'const': '27'}
                                }
                            },
                            'key_value_objs': {
                                'type': 'array',
                                'items': {
                                    'type': 'object',
                                    'properties': {
                                        'key': {'const': 'temp_f'},
                                        'value': {'const': '27'}
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    def test_default_search_sync(self):
        """Test the data release subcommand."""
        main('searchsync', '--objects-per-page', '4', '--threads', '1')
        resp = requests.get('http://localhost:9200/pacifica_search/_stats')
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(
            resp.json()['indices']['pacifica_search']['primaries']['docs']['count'], 45)
        resp = requests.get('http://localhost:9200/pacifica_search/doc/transactions_67')
        self.assertEqual(resp.status_code, 200)
        validate(resp.json(), schema=self.es_schema)

    def test_trans_data_release(self):
        """Test transaction data release."""
        main('--verbose', 'data_release', '--keyword', 'transactions.created',
             '--time-after', '365 days after')
        resp = requests.get('http://localhost:8121/transactions?_id=1234')
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.json()[0]['suspense_date'], '2018-07-15')
        resp = requests.get(
            'http://localhost:8121/transaction_user?transaction=1234')
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.json()[0]['transaction'], 1234)
        resp = requests.get(
            'http://localhost:8121/transaction_user?transaction=1235')
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(len(resp.json()), 0)

    def test_default_data_release(self):
        """Test the data release subcommand."""
        main('data_release', '--time-after',
             '365 days after', '--exclude', u'1234cé')
        resp = requests.get('http://localhost:8121/projects?_id=1234b%C3%A9')
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.json()[0]['suspense_date'], '2017-12-10')
        resp = requests.get(
            'http://localhost:8121/transaction_user?transaction=1234')
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.json()[0]['transaction'], 1234)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""This is the admin main method."""
from __future__ import print_function, absolute_import
import logging
from sys import argv as sys_argv
from argparse import ArgumentParser
from datetime import timedelta
from six import text_type
from .data_release import data_release, VALID_KEYWORDS
from .search_sync import search_sync

logging.basicConfig()
LOGGER = logging.getLogger('urllib3')


def objstr_to_timedelta(obj_str):
    """Turn an object string of the format X unit ago into timedelta."""
    value, unit, check = obj_str.split()
    assert check == 'after' or check == 'ago'
    return timedelta(**{unit: float(value)})


def objstr_to_keyword(obj_str):
    """Verify the obj_str is a valid keyword."""
    assert obj_str in VALID_KEYWORDS
    return obj_str


def datarel_options(datarel_parser):
    """Add data release options to the parser."""
    datarel_parser.add_argument(
        '--exclude', dest='exclude',
        help='id of keyword prefix to exclude.',
        nargs='*', default=set(), type=text_type
    )
    datarel_parser.add_argument(
        '--keyword', dest='keyword', type=objstr_to_keyword,
        help='keyword one of {}.'.format(', '.join(VALID_KEYWORDS)),
        required=False, default=VALID_KEYWORDS[0]
    )
    datarel_parser.add_argument(
        '--time-after', dest='time_after', type=objstr_to_timedelta,
        help='set suspense date on data to X days after keyword.',
        required=False, default=timedelta(days=36500)
    )
    datarel_parser.add_argument(
        '--time-ago', dest='time_ago', type=objstr_to_timedelta,
        help='only objects updated after X days ago.',
        required=False, default=timedelta(days=3650)
    )
    datarel_parser.set_defaults(func=data_release)


def create_subcommands(subparsers):
    """Create the subcommands from the subparsers."""
    datarel_parser = subparsers.add_parser(
        'data_release',
        help='data_release help',
        description='data release by policy'
    )
    searchsync_parser = subparsers.add_parser(
        'searchsync',
        help='searchsync help',
        description='sync sql data to elastic for search'
    )
    return datarel_parser, searchsync_parser


def searchsync_options(searchsync_parser):
    """Add the searchsync command line options."""
    searchsync_parser.add_argument(
        '--objects-per-page', default=40000,
        type=int, help='objects per bulk upload.',
        required=False, dest='items_per_page'
    )
    searchsync_parser.add_argument(
        '--threads', default=4, required=False,
        type=int, help='number of threads to sync data',
    )
    searchsync_parser.add_argument(
        '--time-ago', dest='time_ago', type=objstr_to_timedelta,
        help='only objects newer than X days ago.', required=False,
        default=timedelta(days=36500)
    )
    searchsync_parser.set_defaults(func=search_sync)


def main(*argv):
    """Main method for admin command line tool."""
    parser = ArgumentParser()
    parser.add_argument(
        '--verbose', default=False, action='store_true',
        help='enable verbose debug output'
    )
    subparsers = parser.add_subparsers(help='sub-command help')
    datarel_parser, searchsync_parser = create_subcommands(
        subparsers
    )
    searchsync_options(searchsync_parser)
    datarel_options(datarel_parser)
    if not argv:  # pragma: no cover
        argv = sys_argv[1:]
    args = parser.parse_args(argv)
    if args.verbose:
        LOGGER.setLevel('DEBUG')
    args.func(args)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""CherryPy Status Policy object class."""
import requests
from cherrypy import tools, HTTPError
from pacifica.policy.validation import validate_project
from pacifica.policy.config import get_config
from pacifica.policy.status.base import QueryBase


# pylint: disable=too-few-public-methods
class InstrumentsByProject(QueryBase):
    """Retrieves instrument list for a given project."""

    exposed = True

    @staticmethod
    def _get_instruments_for_project(project_id):
        """Return a list with all the instruments belonging to this project."""
        md_url = u'{0}/projectinfo/by_project_id/{1}'.format(
            get_config().get('metadata', 'endpoint_url'), project_id
        )
        query = requests.get(url=md_url)
        if query.status_code == 200:
            return query.json()
        raise HTTPError('404 Not Found')

    # CherryPy requires these named methods
    # Add HEAD (basically Get without returning body
    # pylint: disable=invalid-name
    @staticmethod
    @tools.json_out()
    @validate_project()
    def GET(project_id=None):
        """CherryPy GET method."""
        project_info = InstrumentsByProject._get_instruments_for_project(
            project_id)
        instruments = {index: info for (
            index, info) in project_info.get('instruments').items()}
        cleaned_instruments = []
        if instruments:
            clean_info = {
                'id': -1,
                'text': u'All Available Instruments for Project {0}'.format(project_id),
                'name': 'All Instruments',
                'active': 'Y'
            }
            cleaned_instruments.append(clean_info)

        for inst_id in instruments.keys():
            info = instruments.get(inst_id)
            clean_info = {
                'id': inst_id,
                'text': u'Instrument {0}: {1}'.format(inst_id, info.get('display_name')),
                'name': info.get('name_short'),
                'active': 'Y' if info.get('active') else 'N'
            }
            cleaned_instruments.append(clean_info)

        return_block = {
            'total_count': len(cleaned_instruments),
            'incomplete_results': False,
            'items': cleaned_instruments
        }
        return return_block
# pylint: enable=too-few-public-methods



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""CherryPy Status Policy object class."""
from pacifica.policy.status.transaction.search import TransactionSearch
from pacifica.policy.status.transaction.lookup import TransactionLookup
from pacifica.policy.status.transaction.files import FileLookup


# pylint: disable=too-few-public-methods
class TransactionQuery(object):
    """CherryPy root object class."""

    exposed = False

    def __init__(self):
        """Create local objects for sub tree items."""
        self.search = TransactionSearch()
        self.by_id = TransactionLookup()
        self.files = FileLookup()
# pylint: enable=too-few-public-methods



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Search transaction rendering methods."""
from six import text_type
from .base import SearchBase


class FilesRender(SearchBase):
    """Render an insitution for search."""

    fields = [
        'obj_id', 'display_name', 'mtime', 'ctime', 'keyword',
        'name', 'subdir', 'size', 'hashsum', 'hashtype',
        'updated_date', 'created_date'
    ]

    @staticmethod
    def obj_id(**file_obj):
        """Return string for object id."""
        return text_type('files_{_id}').format(**file_obj)

    @staticmethod
    def updated_date(**file_obj):
        """Return string for the updated date."""
        return text_type('{updated}').format(**file_obj)

    @staticmethod
    def created_date(**file_obj):
        """Return string for the created date."""
        return text_type('{created}').format(**file_obj)

    @staticmethod
    def display_name(**file_obj):
        """Return the string to render display_name."""
        return text_type('{subdir}/{name}').format(**file_obj)

    @staticmethod
    def mtime(**file_obj):
        """Return the string to render mtime."""
        return text_type('{mtime}').format(**file_obj)

    @staticmethod
    def ctime(**file_obj):
        """Return the string to render mtime."""
        return text_type('{ctime}').format(**file_obj)

    @staticmethod
    def name(**file_obj):
        """Return the string to render mtime."""
        return text_type('{name}').format(**file_obj)

    @staticmethod
    def subdir(**file_obj):
        """Return the string to render mtime."""
        return text_type('{subdir}').format(**file_obj)

    @staticmethod
    def size(**file_obj):
        """Return the string to render size."""
        return text_type('{size}').format(**file_obj)

    @staticmethod
    def hashsum(**file_obj):
        """Return the string to render hashsum."""
        return text_type('{hashsum}').format(**file_obj)

    @staticmethod
    def hashtype(**file_obj):
        """Return the string to render hashtype."""
        return text_type('{hashtype}').format(**file_obj)

    @staticmethod
    def keyword(**file_obj):
        """Return the rendered string for keywords."""
        return text_type(' ').join(text_type('{subdir}/{name}').format(**file_obj).split('/'))

    # this is needed because it is abastract but it's never called
    @classmethod
    def get_transactions(cls, **_kwargs):  # pragma: no cover
        """Just return empty for this."""
        return []



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Search transaction rendering methods."""
from six import text_type
from .base import SearchBase


class UsersRender(SearchBase):
    """Render a transaction for search."""

    fields = [
        'obj_id', 'display_name', 'keyword', 'release',
        'updated_date', 'created_date'
    ]

    @staticmethod
    def obj_id(**user_obj):
        """Return string for object id."""
        return text_type('users_{_id}').format(**user_obj)

    @staticmethod
    def updated_date(**user_obj):
        """Return string for the updated date."""
        return text_type('{updated}').format(**user_obj)

    @staticmethod
    def created_date(**user_obj):
        """Return string for the created date."""
        return text_type('{created}').format(**user_obj)

    @staticmethod
    def display_name(**user_obj):
        """Return the string to render display_name."""
        return text_type('{last_name}, {first_name} {middle_initial}').format(**user_obj)

    @staticmethod
    def keyword(**user_obj):
        """Return the rendered string for keywords."""
        return text_type('{last_name}, {first_name} {middle_initial}').format(**user_obj)

    @classmethod
    def release(cls, **user_obj):
        """Return whether the user has released anything."""
        for trans_id in cls._transsip_transsap_merge({'submitter': user_obj['_id']}, '_id'):
            if cls.get_rel_by_args('transaction_user', transaction=trans_id, relationship=cls.releaser_uuid):
                return 'true'
        return 'false'

    @classmethod
    def get_transactions(cls, **user_obj):
        """Return the list of transaction ids for the user."""
        return [
            'transactions_{}'.format(trans_id)
            for trans_id in cls._transsip_transsap_merge({'submitter': user_obj['_id']}, '_id')
        ]



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Search transaction rendering methods."""
from six import text_type
from ..config import get_config
from .users import UsersRender
from .institutions import InstitutionsRender
from .instruments import InstrumentsRender
from .groups import GroupsRender
from .projects import ProjectsRender
from .science_themes import ScienceThemesRender
from .keys import KeysRender
from .values import ValuesRender
from .files import FilesRender
from .base import SearchBase


class TransactionsRender(SearchBase):
    """Render a transaction for search."""

    fields = [
        'obj_id', 'access_url', 'has_doi', 'release',
        'updated_date', 'created_date', 'description'
    ]
    rel_objs = [
        'users', 'institutions', 'instruments', 'groups',
        'projects', 'science_themes', 'key_value_pairs', 'files'
    ]

    @staticmethod
    def obj_id(**trans_obj):
        """Return string for object id."""
        return text_type('transactions_{_id}').format(**trans_obj)

    @staticmethod
    def description(**trans_obj):
        """Return string for the updated date."""
        return text_type('{description}').format(**trans_obj)

    @staticmethod
    def updated_date(**trans_obj):
        """Return string for the updated date."""
        return text_type('{updated}').format(**trans_obj)

    @staticmethod
    def created_date(**trans_obj):
        """Return string for the created date."""
        return text_type('{created}').format(**trans_obj)

    @classmethod
    def release(cls, **trans_obj):
        """Return 'true' if transaction has been release."""
        if cls.get_rel_by_args('transaction_user', transaction=trans_obj['_id'], relationship=cls.releaser_uuid):
            return 'true'
        return 'false'

    @classmethod
    def access_url(cls, **trans_obj):
        """Figure out the access url for the transaction."""
        trans_doi = cls.get_trans_doi(trans_obj['_id'])
        if trans_doi != 'false':
            trans_obj['doi'] = trans_doi
            return get_config().get('policy', 'doi_url_format').format(**trans_obj)
        if cls.release(**trans_obj) == 'true':
            return get_config().get('policy', 'release_url_format').format(**trans_obj)
        return get_config().get('policy', 'internal_url_format').format(**trans_obj)

    @classmethod
    def get_trans_doi(cls, trans_id):
        """Get the transaction doi or return false."""
        trans_user = cls.get_rel_by_args('transaction_user', transaction=trans_id, relationship=cls.releaser_uuid)
        if not trans_user:
            return 'false'
        rel_uuid = trans_user[0].get('uuid')
        doi_trans = cls.get_rel_by_args('doi_transaction', transaction=rel_uuid)
        if doi_trans:
            return doi_trans[0].get('doi', 'false')
        return 'false'

    @classmethod
    def has_doi(cls, **trans_obj):
        """Return boolean if the transaction has a DOI."""
        if cls.get_trans_doi(trans_obj['_id']) != 'false':
            return 'true'
        return 'false'

    @classmethod
    def files_obj_lists(cls, **trans_obj):
        """Get the projects related to the transaction."""
        return [
            FilesRender.render(file_obj)
            for file_obj in cls.get_rel_by_args('files', transaction=trans_obj['_id'])
        ]

    @classmethod
    def users_obj_lists(cls, **trans_obj):
        """Get the user objects and relationships."""
        ret = {'submitter': []}
        for user_id in cls._transsip_transsap_merge({'_id': trans_obj['_id']}, 'submitter'):
            ret['submitter'].append(
                UsersRender.render(cls.get_rel_by_args('users', _id=user_id)[0])
            )
        for trans_user_obj in cls.get_rel_by_args('transaction_user', transaction=trans_obj['_id']):
            rel_obj = cls.get_rel_by_args('relationships', uuid=trans_user_obj['relationship'])[0]
            rel_list = ret.get(rel_obj['name'], [])
            rel_list.append(
                UsersRender.render(cls.get_rel_by_args('users', _id=trans_user_obj['user'])[0])
            )
            ret[rel_obj['name']] = rel_list
        return ret

    @classmethod
    def institutions_obj_lists(cls, **trans_obj):
        """Get the institutions related to the transaction."""
        ret = set()
        user_id_list = cls._transsip_transsap_merge({'_id': trans_obj['_id']}, 'submitter')
        user_id_list.update([obj['user'] for obj in cls.get_rel_by_args(
            'transaction_user', transaction=trans_obj['_id'])])
        for user_id in user_id_list:
            for inst_obj in cls.get_rel_by_args('institution_user', user=user_id):
                ret.update([inst_obj['institution']])
        return [
            InstitutionsRender.render(
                cls.get_rel_by_args('institutions', _id=inst_id)[0]
            ) for inst_id in ret
        ]

    @classmethod
    def instruments_obj_lists(cls, **trans_obj):
        """Get the instruments related to the transaction."""
        ret = set()
        for trans_obj in cls.get_rel_by_args('transsip', _id=trans_obj['_id']):
            ret.update([trans_obj['instrument']])
        return [
            InstrumentsRender.render(
                cls.get_rel_by_args('instruments', _id=inst_id)[0]
            ) for inst_id in ret
        ]

    @classmethod
    def groups_obj_lists(cls, **trans_obj):
        """Get the instrument groups related to the transaction."""
        ret = set()
        for inst_obj in cls.get_rel_by_args('transsip', _id=trans_obj['_id']):
            for group_obj in cls.get_rel_by_args('instrument_group', instrument=inst_obj['instrument']):
                ret.update([group_obj['group']])
        return [
            GroupsRender.render(
                cls.get_rel_by_args('groups', _id=group_id)[0]
            ) for group_id in ret
        ]

    @classmethod
    def projects_obj_lists(cls, **trans_obj):
        """Get the projects related to the transaction."""
        return [
            ProjectsRender.render(
                cls.get_rel_by_args('projects', _id=proj_id)[0]
            ) for proj_id in cls._transsip_transsap_merge({'_id': trans_obj['_id']}, 'project')
        ]

    @classmethod
    def science_themes_obj_lists(cls, **trans_obj):
        """Get science themes related to the transaction."""
        return [
            ScienceThemesRender.render(
                cls.get_rel_by_args('projects', _id=proj_id)[0]
            ) for proj_id in cls._transsip_transsap_merge({'_id': trans_obj['_id']}, 'project')
        ]

    @classmethod
    def key_value_pairs_obj_lists(cls, **trans_obj):
        """Get the key value pairs related to the transaction."""
        ret = {
            'key_value_hash': {},
            'key_objs': [],
            'value_objs': []
        }
        for trans_kvp_obj in cls.get_rel_by_args('trans_key_value', transaction=trans_obj['_id']):
            key_obj = cls.get_rel_by_args('keys', _id=trans_kvp_obj['key'])[0]
            value_obj = cls.get_rel_by_args('values', _id=trans_kvp_obj['value'])[0]
            ret['key_objs'].append(KeysRender.render(key_obj))
            ret['value_objs'].append(ValuesRender.render(value_obj))
            ret['key_value_hash'][key_obj['key']] = value_obj['value']
        return ret

    # this doesn't get executed but is needed to be here to satisfy abstract method
    @classmethod
    def get_transactions(cls, **_trans_obj):  # pragma: no cover
        """Just return an empty list."""
        return []



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Sync the database to elasticsearch index for use by Searching tools."""
from __future__ import print_function, absolute_import
from json import dumps
from time import sleep
from threading import Thread
try:
    from Queue import Queue
except ImportError:  # pragma: no cover
    from queue import Queue
from math import ceil
from datetime import datetime
import requests
from six import text_type
from elasticsearch import Elasticsearch, helpers, ElasticsearchException
from .config import get_config
from .root import Root
from .search_render import ELASTIC_INDEX, SearchRender

ELASTIC_CONNECT_ATTEMPTS = 40
ELASTIC_WAIT = 3
ELASTIC_ENDPOINT = get_config().get('elasticsearch', 'url')
SYNC_OBJECTS = [
    'keys',
    'values',
    'relationships',
    'transactions',
    'projects',
    'users',
    'instruments',
    'institutions',
    'groups'
]


def es_client():
    """Get the elasticsearch client object."""
    es_kwargs = {}
    if get_config().getboolean('elasticsearch', 'sniff'):
        es_kwargs['sniff_on_start'] = True
        es_kwargs['sniff_on_connection_fail'] = True
        es_kwargs['sniff_timeout'] = get_config().getint('elasticsearch', 'timeout')
    es_kwargs['timeout'] = get_config().getint('elasticsearch', 'timeout')
    esclient = Elasticsearch(
        [ELASTIC_ENDPOINT],
        **es_kwargs
    )
    mapping_params = {
        'properties': {
            'key': {'type': 'keyword'},
            'value': {'type': 'keyword'},
            'key_value_pairs': {
                'properties': {
                    'key': {'type': 'keyword'},
                    'value': {'type': 'keyword'}
                }
            },
            'release': {
                'type': 'keyword'
            },
            'updated_date': {
                'type': 'date'
            },
            'created_date': {
                'type': 'date'
            },
            'transaction_ids': {
                'type':     'text',
                'fielddata': True
            },
            'has_doi': {
                'type':      'text',
                'fielddata': True
            },
            'users': {
                'properties': {
                    'keyword': {
                        'type': 'keyword'
                    },

                }
            },
            'instruments': {
                'properties': {
                    'keyword': {
                        'type': 'keyword'
                    },

                }
            },
            'projects': {
                'properties': {
                    'keyword': {
                        'type': 'keyword'
                    },
                    'closed_date': {
                        'type': 'date'
                    },
                    'actual_start_date': {
                        'type': 'date'
                    },
                    'actual_end_date': {
                        'type': 'date'
                    }
                }
            },
            'institutions': {
                'properties': {
                    'keyword': {
                        'type': 'keyword'
                    },

                }
            },
            'science_themes': {
                'properties': {
                    'keyword': {
                        'type': 'keyword'
                    },

                }
            },
            'instrument_groups': {
                'properties': {
                    'keyword': {
                        'type': 'keyword'
                    },
                }
            }
        }
    }
    # pylint: disable=unexpected-keyword-arg
    esclient.indices.create(index=ELASTIC_INDEX, ignore=400)
    esclient.indices.put_mapping(
        index=ELASTIC_INDEX,
        doc_type='doc',
        body=dumps(mapping_params)
    )
    # pylint: enable=unexpected-keyword-arg
    return esclient


def try_es_connect(attempts=0):
    """Recursively try to connect to elasticsearch."""
    try:
        cli = es_client()
        cli.info()
    except ElasticsearchException as ex:  # pragma: no cover pulled from metadata
        if attempts < ELASTIC_CONNECT_ATTEMPTS:
            sleep(ELASTIC_WAIT)
            attempts += 1
            try_es_connect(attempts)
        else:
            raise ex


def start_work(work_queue):
    """The main thread for the work."""
    cli = es_client()
    job = work_queue.get()
    while job:
        print('Starting {} ({}): {}'.format(job[0], job[1], job[2]))
        try_doing_work(cli, job)
        work_queue.task_done()
        print('Finished {} ({}): {}'.format(job[0], job[1], job[2]))
        job = work_queue.get()
    work_queue.task_done()


def try_doing_work(cli, job):
    """Try doing some work even if you fail."""
    tries_left = 5
    success = False
    while not success and tries_left:
        try:
            helpers.bulk(cli, yield_data(*job))
            success = True
        except ElasticsearchException:  # pragma: no cover
            tries_left -= 1
    return success


def yield_data(obj, time_field, page, items_per_page, time_delta):
    """yield objects from obj for bulk ingest."""
    get_args = {
        '{time_field}': '{epoch}',
        '{time_field}_operator': 'gt',
        'page_number': '{page}',
        'items_per_page': '{items_per_page}',
        'recursion_depth': '0',
        'recursion_limit': '1'
    }
    get_list = ['{}={}'.format(key, val) for key, val in get_args.items()]
    url = '{base_url}/{orm_obj}?'+'&'.join(get_list)
    resp = requests.get(
        text_type(url).format(
            base_url=get_config().get('metadata', 'endpoint_url'),
            orm_obj=obj,
            time_field=time_field,
            epoch=time_delta.isoformat(),
            page=page,
            items_per_page=items_per_page
        )
    )
    objs = resp.json()
    return SearchRender.generate(obj, objs)


def create_worker_threads(threads, work_queue):
    """Create the worker threads and return the list."""
    work_threads = []
    for i in range(threads):
        wthread = Thread(target=start_work, args=(work_queue,))
        wthread.daemon = True
        wthread.start()
        work_threads.append(wthread)
    return work_threads


def generate_work(items_per_page, work_queue, time_ago):
    """Generate the work from the db and send it to the work queue."""
    now = datetime.now()
    time_delta = (now - time_ago).replace(microsecond=0)
    for obj in SYNC_OBJECTS:
        for time_field in ['created', 'updated']:
            resp = requests.get(
                text_type('{base_url}/objectinfo/{orm_obj}?{time_field}={epoch}&{time_field}_operator=gt').format(
                    base_url=get_config().get('metadata', 'endpoint_url'),
                    time_field=time_field,
                    orm_obj=obj,
                    epoch=time_delta.isoformat()
                )
            )
            total_count = resp.json()['record_count']
            num_pages = int(ceil(float(total_count) / items_per_page))
            for page in range(1, num_pages + 1):
                work_queue.put(
                    (obj, time_field, page, items_per_page, time_delta))


def search_sync(args):
    """Main search sync subcommand."""
    try_es_connect()
    Root.try_meta_connect()
    work_queue = Queue(32)
    work_threads = create_worker_threads(args.threads, work_queue)
    generate_work(args.items_per_page, work_queue, args.time_ago)
    for i in range(args.threads):
        work_queue.put(False)
    for wthread in work_threads:
        wthread.join()
    work_queue.join()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""CherryPy Status Policy object class."""
from json import loads
from cherrypy import tools, HTTPError
import requests
from pacifica.policy.config import get_config
from pacifica.policy.status.base import QueryBase


# pylint: disable=too-few-public-methods
class InstrumentKeywordSearch(QueryBase):
    """Retrieves a set of projects for a given keyword set."""

    exposed = True

    def _get_instruments_for_keywords(self, user_id, search_terms=''):
        """Return a list with all the instruments having this term."""
        # is_admin = self._is_admin(user_id)
        inst_list_url = '{0}/instrumentinfo/search/{1}'.format(
            get_config().get('metadata', 'endpoint_url'), search_terms
        )
        inst_query = requests.get(url=inst_list_url)
        if inst_query.status_code == 200:
            inst_response = loads(inst_query.text)
            return self._clean_up_instrument_list(inst_response, user_id)
        else:
            raise HTTPError(inst_query.status_code)

    def _clean_up_instrument_list(self, inst_response, user_id):
        """Clear out entries that done belong to this user."""
        is_admin = self._is_admin(user_id)
        output_list = []
        if not is_admin:
            inst_for_user_url = '{0}/instrumentinfo/by_user_id/{1}'.format(
                get_config().get('metadata', 'endpoint_url'), user_id
            )
            query = requests.get(url=inst_for_user_url)
            response = loads(query.text)
            output_list = InstrumentKeywordSearch._squash_output_list(
                response, inst_response)
        else:
            output_list = inst_response

        return output_list

    @staticmethod
    def _squash_output_list(inst_for_user_list, full_inst_list):
        """Filter entries in the full instrument list."""
        inst_list = [i['id'] for i in inst_for_user_list]
        output_list = [obj for obj in full_inst_list if obj['id'] in inst_list]

        return output_list

    # CherryPy requires these named methods
    # Add HEAD (basically Get without returning body
    # pylint: disable=invalid-name
    @tools.json_out()
    def GET(self, search_terms='', **kwargs):
        """CherryPy GET method."""
        user_id = kwargs['user'] if 'user' in kwargs else None
        return self._get_instruments_for_keywords(user_id, search_terms)
# pylint: enable=too-few-public-methods



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""CherryPy Status Metadata object class."""
import requests
from cherrypy import tools, request
from pacifica.policy.validation import validate_user
from pacifica.policy.config import get_config
from pacifica.policy.reporting.transaction.query_base import QueryBase


# pylint: disable=too-few-public-methods
class TransactionDetails(QueryBase):
    """Retrieves a list of all transactions matching the search criteria."""

    exposed = True

    @staticmethod
    def _get_transaction_list_details(transaction_list, user_id):
        header_list = {'Content-Type': 'application/json'}
        req = requests.post(
            url='{0}/summaryinfo/transaction_details'.format(
                get_config().get('metadata', 'endpoint_url')
            ),
            json=transaction_list,
            headers=header_list
        )

        user_info = QueryBase._merge_two_dicts(
            QueryBase.base_user_info,
            QueryBase.get_full_user_info(user_id)
        )

        results = {}
        req_json = req.json()
        for transaction_id in req_json.keys():
            info = req_json[transaction_id]
            if user_info['emsl_employee'] or (
                    info['project_id'] in user_info['project_list'] and
                    info['instrument_id'] in user_info['instrument_list']
            ):
                results[transaction_id] = info
        return results

    # CherryPy requires these named methods
    # pylint: disable=invalid-name
    @staticmethod
    @tools.json_in()
    @tools.json_out()
    @validate_user()
    def POST(user_id=None):
        """CherryPy GET method."""
        return TransactionDetails._get_transaction_list_details(request.json, user_id)
# pylint: enable=too-few-public-methods



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""CherryPy Status Policy object class."""
from pacifica.policy.status.instrument.by_project_id import InstrumentsByProject
from pacifica.policy.status.instrument.search import InstrumentKeywordSearch


# pylint: disable=too-few-public-methods
class InstrumentQuery(object):
    """CherryPy root object class."""

    exposed = False

    def __init__(self):
        """Create local objects for sub tree items."""
        # self.search = ProjectSearch()
        self.by_project_id = InstrumentsByProject()
        self.search = InstrumentKeywordSearch()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Search base class has some common data and logic."""
try:
    from functools import lru_cache
except ImportError:  # pragma: no cover only python 2
    from backports.functools_lru_cache import lru_cache
from functools import wraps
from six import text_type
import requests
from ..admin import AdminPolicy
from ..config import get_config


_LRU_GLOBAL_ARGS = {
    'maxsize': get_config().getint('policy', 'cache_size'),
    'typed': False
}


def search_lru_cache(func):
    """Wrap a function with my lru cache args."""
    @lru_cache(**_LRU_GLOBAL_ARGS)
    @wraps(func)
    def wrapper(*args, **kwargs):
        """Calling inter function."""
        return func(*args, **kwargs)
    return wrapper


class SearchBase(object):
    """Search base class containing common data and logic."""

    fields = []
    rel_objs = []
    obj_type = 'unimplemented'
    releaser_uuid = AdminPolicy().get_relationship_info(name='authorized_releaser')[0].get('uuid')
    search_required_uuid = AdminPolicy().get_relationship_info(name='search_required')[0].get('uuid')
    global_get_args = {
        'recursion_depth': '0',
        'recursion_limit': '1'
    }

    @classmethod
    @search_lru_cache
    def get_rel_by_args(cls, mdobject, **kwargs):
        """Get the transaction user relationship."""
        get_params = kwargs.copy()
        get_params.update(cls.global_get_args)
        resp = requests.get(
            text_type('{base_url}/{mdobject}').format(
                mdobject=mdobject,
                base_url=get_config().get('metadata', 'endpoint_url')
            ),
            params=get_params
        )
        assert resp.status_code == 200
        return resp.json()

    @classmethod
    def get_transactions(cls, **kwargs):  # pragma: no cover abstract method
        """Unimplemented in the base class."""
        raise NotImplementedError

    @classmethod
    def _transsip_transsap_merge(cls, trans_obj, key):
        ret = set()
        for trans_rel in ['transsip', 'transsap']:
            for rel_trans_obj in cls.get_rel_by_args(trans_rel, **trans_obj):
                ret.update([rel_trans_obj[key]])
        return ret

    @staticmethod
    def _cls_name_to_module(rel_cls):
        """Convert the class name to the object type and module to load."""
        return rel_cls.__module__.split('.')[-1]

    @classmethod
    def render(cls, obj, render_rel_objs=False, render_trans_ids=False):
        """Render the object and return it."""
        ret = {'type': cls._cls_name_to_module(cls)}
        for key in cls.fields:
            ret[key] = getattr(cls, key)(**obj)
        if render_rel_objs:
            for related_obj_name in cls.rel_objs:
                ret[related_obj_name] = getattr(cls, '{}_obj_lists'.format(related_obj_name))(**obj)
        if render_trans_ids:
            ret['transaction_ids'] = cls.get_transactions(**obj)
        return ret
